/*
 * Created by Brett on 17/11/23.
 * Licensed under GNU General Public License V3.0
 * See LICENSE file for license detail
 */
#include <bf_tokenizer.h>
#include <sstream>
#include <random>

std::vector<token_t> tokenize(const std::string& program)
{
    std::vector<token_t> tokens;
    
    size_t index = 0;
    while (index < program.size())
    {
        auto c = program[index];
        switch (c)
        {
            case '>':
                tokens.emplace_back(bf_token::INC_DP);
                break;
            case '<':
                tokens.emplace_back(bf_token::DEC_DP);
                break;
            case '+':
                tokens.emplace_back(bf_token::INC_DV);
                break;
            case '-':
                tokens.emplace_back(bf_token::DEC_DV);
                break;
            case '.':
                tokens.emplace_back(bf_token::PRINT);
                break;
            case ',':
                tokens.emplace_back(bf_token::READ);
                break;
            case '[':
                tokens.emplace_back(bf_token::OPEN);
                break;
            case ']':
                tokens.emplace_back(bf_token::CLOSE);
                break;
            default:
                break;
        }
        index++;
    }
    
    return tokens;
}

std::string generateName()
{
    std::stringstream ss;
    ss << std::hex;
    std::random_device rd;
    std::seed_seq seed{rd(), rd(), rd(), rd()};
    std::mt19937_64 gen(seed);
    std::uniform_int_distribution<int> dis(0, 15);
    
    for (int i = 0; i < 24; i++)
        ss << dis(gen);
    
    return ss.str();
}

std::vector<token_t>& bf_name(std::vector<token_t>& tokens)
{
    size_t search_index = 0;
    while (search_index < tokens.size())
    {
        if (tokens[search_index].token == bf_token::OPEN)
        {
            auto name = generateName();
            size_t sp = 1;
            size_t search_2 = search_index;
            tokens[search_index].name = name;
            while (++search_2 < tokens.size())
            {
                if (tokens[search_2].token == bf_token::OPEN)
                    sp++;
                if (tokens[search_2].token == bf_token::CLOSE)
                    sp--;
                if (sp == 0)
                    break;
            }
            tokens[search_2].name = name;
        }
        search_index++;
    }
    return tokens;
}
